From aff741ccbbb9ad06fefe12f4be90372e14baa834 Mon Sep 17 00:00:00 2001
From: Sebastian Poeplau <sebastian.poeplau@eurecom.fr>
Date: Mon, 25 Feb 2019 13:32:53 +0100
Subject: [PATCH 2/2] Handle munmap

The implementation ignores the length parameter to munmap, assuming it to be
always exactly the size that mmap was called with.
---
 lib/Core/MemoryManager.cpp          | 8 ++++++--
 lib/Core/SpecialFunctionHandler.cpp | 1 +
 2 files changed, 7 insertions(+), 2 deletions(-)

diff --git a/lib/Core/MemoryManager.cpp b/lib/Core/MemoryManager.cpp
index 5d8eeac..cdcd4f7 100644
--- a/lib/Core/MemoryManager.cpp
+++ b/lib/Core/MemoryManager.cpp
@@ -194,8 +194,12 @@ void MemoryManager::deallocate(const MemoryObject *mo) { assert(0); }
 
 void MemoryManager::markFreed(MemoryObject *mo) {
   if (objects.find(mo) != objects.end()) {
-    if (!mo->isFixed && !DeterministicAllocation)
-      free((void *)mo->address);
+    if (!DeterministicAllocation) {
+      if (mo->isFixed)
+        munmap((void *)mo->address, mo->size);
+      else
+        free((void *)mo->address);
+    }
     objects.erase(mo);
   }
 }
diff --git a/lib/Core/SpecialFunctionHandler.cpp b/lib/Core/SpecialFunctionHandler.cpp
index c1722e2..074e449 100644
--- a/lib/Core/SpecialFunctionHandler.cpp
+++ b/lib/Core/SpecialFunctionHandler.cpp
@@ -79,6 +79,7 @@ static SpecialFunctionHandler::HandlerInfo handlerInfo[] = {
   addDNR("klee_report_error", handleReportError),
   add("calloc", handleCalloc, true),
   add("free", handleFree, false),
+  add("munmap", handleFree, false), // ignore length parameter
   add("klee_assume", handleAssume, false),
   add("klee_check_memory_access", handleCheckMemoryAccess, false),
   add("klee_get_valuef", handleGetValue, true),
-- 
2.23.0

